@@ -89,5 +89,5 @@ plot.tind=function(x,...)
   lines(0:x$inputs$max.score,x$est$test$expected,col="gray",pch=19,cex=0.7)
   lines(0:x$inputs$max.score,x$inputs$observed_smooth,col="lightgreen")
   legend("topleft", legend = c("observed", "expected"), bty = "n",
-         lwd = 1, cex = 0.7, col = c("#4DAF4A", "gray"), lty = c(NA,1), pch = c(19,NA))
+         lwd = 1, cex = 0.7, col = c("#4DAF4A", "gray"), lty = c(NA,1), pch = c(19,NA),inset=0.01)
 }
\ No newline at end of file
